//@line 37 "/home/ondrejd/mozilla/extensions/xpcomviewer/resources/content/xpcomviewer.js"


//////////////////////////////////////////////////////////////////////////
// Global objects/variables

/** String bundles */
var gStrBundle = null;
var gBrandStrBundle = null;


/**
 * Bookmarks related variables
 */
var gBookmarksSrv = null;
var gAllowBookmarks = false;


/**
 * Variables that holds user preferences that modified XPCOMViewer behaviour
 */
var gStartupTreeViewName = "";


/**
 * Holds pointers to main XPCOMViewer's UI elements
 */
var gMainContent = null;
var gMainContentTabbox = null;
var gFilterToolbarTextbox = null;
var gBookmarksTreeSplitter = null;
var gStatusbarLabel = null;
var gBookmaksToolbarbutton = null;
var gBookmaksToolbarbuttonSeparator = null;
var gBookmarksFilterToolbarbutton = null;
var gBookmarksFilterToolbarbuttonSep = null;
var gBookmarksMainTreeCol = null;
var gBookmarksMainTreeColSep = null;
var gXpcomTreeTab = null;
var gLxrBrowserTab = null;
var gLxrBrowserCmdMenuitem = null;
var gLxrBrowserCmdPopupitem = null;
var gBookmarksPopupitem1 = null;
var gBookmarksPopupitem2 = null;
var gBookmarksPopupSeparator = null;
var gBookmarksTogglePaneMenuitem = null;
var gBookmarksTogglePaneMenuitemSep = null;


//////////////////////////////////////////////////////////////////////////
// Global XPCOMViewer's tree variables and objects

/**
 * These variables are used by all tree views.
 * For easy creating new treeview us newTreeView object prototype
 * and for treeitems use fucntion newTreeItem which creates
 * new treeitem object for you.
 *
 * @todo Make prototype of TreeView and share it between views too
 * @todo Tree sorting
 */

// "System" names of single tree views
const kTreeViewsNames = { classes : "Components.classes",
                          ifaces  : "Components.interfaces",
                          results : "Components.results" };

// "Human" names of single tree views (are loaded in gXpcomViewer.onLoad()
// from string bundles.
var kTreeViewsItemNames = { classes : "",
                            ifaces  : "",
                            results : "" };

// Some constants for gWebProgressListener
const NOTIFY_STATE_DOCUMENT = Components.interfaces.nsIWebProgress.NOTIFY_STATE_DOCUMENT;
const STATE_IS_DOCUMENT = Components.interfaces.nsIWebProgressListener.STATE_IS_DOCUMENT;
const STATE_START = Components.interfaces.nsIWebProgressListener.STATE_START;
const STATE_STOP = Components.interfaces.nsIWebProgressListener.STATE_STOP;


/** 
 * Holds pointer to xpcomviewer tree 
 */
var gXpcomTree = null;

/** 
 * Holds name of currently displayed treeview 
 */
var gCurrentTreeView = "";

/** 
 * Object that holds backups of all three views 
 */
var gXpcomTreeViewBackup = { classes : [],
                             ifaces  : [],
                             results : [] };

/** 
 * Type of used filter 
 */
var gXpcomTreeFilter = false;

/** 
 * Holds last treeview (before apply filter) 
 */
var gXpcomTreeFilterBackup = [];

/** 
 * Holds informations about if tree is sorted and how. Sort direction holds 
 * -1 for descending direction, 0 for natural direction, 1 for ascending.
 */
var gXpcomTreeSort = 
{
  sortedCol : "", // Contains name of currently sorted column
  direction : 0,  // -1 is descending, 0 is natural, 1 is ascending
  fastIndex : 0,  // Holds count of tree items to examine if tree's row count 
                  // is changed from last sort (if not we can use simple array 
                  // reverse)
  
  /**
   * Method for sorting itself.
   */
  sort : function(aX, aY)
  {
    var xText = aX.mColumns[gXpcomTreeSort.sortedCol].mCellText.toLowerCase();
    var yText = aY.mColumns[gXpcomTreeSort.sortedCol].mCellText.toLowerCase();
    
    if(xText > yText) return 1;
    if(xText < yText) return -1;
    return 0;
  }
};


/**
 * Create new selected item object
 */
function SelectedTreeItemPrototype(aIndex, aName, aType)
{
  this.mIndex = aIndex;
  this.mName = aName;
  this.mType = aType;
}


/**
 * Create new tree item
 */
function XpcomTreeItemPrototype(aName, aValue, aType, aLevel, aIsContainer)
{
  var item = new TreeViewItemPrototype(
  		[
       { mColId : "bookmarkCol", mColText : ""},
       { mColId : "nameCol", mColText : aName},
  		 { mColId : "valueCol", mColText : aValue}
      ], 
  		 aLevel, aIsContainer);
  item.mType = aType;

	return item;
} // End of XpcomTreeItemPrototype ();



/**
 * Prototype object for new view of main XPCOMViewer tree. From this prototype 
 * are derived target trees for Components.classes, Components.interfaces 
 * and Components.results.
 * @see global.js for TreeViewPrototype implementation
 * @param  array  Array of treeview items 
 */
function XpcomTreeViewPrototype(aTreeView)
{
  var treeview = new TreeViewPrototype(aTreeView);
  
  treeview.getCellProperties = function (aRow, aCol, aProp) 
  {
    if(!(aRow in this.__treeView__)) return;
    
    var name = this.__treeView__[aRow].mColumns["nameCol"].mCellText;
    
    if(aCol.id == "bookmarkCol")
    {
      if(gBookmarksSrv.hasItemBookmark(name)) {
        aProp.AppendElement(kAtomSrv.getAtom("hasBookmark"));
      } else {
        aProp.AppendElement(kAtomSrv.getAtom("noBookmark"));
      }
    }
    else if(aCol.id == "nameCol") 
    {
      if(this.__treeView__[aRow].mType == "interface" && 
         (name == "nsISupports" || 
          name == "nsIClassInfo" || 
          name == "IDispatch"))
      {
        aProp.AppendElement(kAtomSrv.getAtom("constIface"));
      }
      else if (this.__treeView__[aRow].mType == "method" && 
               name == "QueryInterface")
      {
        aProp.AppendElement(kAtomSrv.getAtom("constMethod"));
      }
      else
        aProp.AppendElement(kAtomSrv.getAtom(this.__treeView__[aRow].mType));
    }
    else
    {
      aProp.AppendElement(kAtomSrv.getAtom("noIcon"));
    }
  };
    
  treeview.cycleHeader = function (aCol)
  {
    var index = gXpcomTree.currentIndex;
    
    // We have to ensure that all containers are closed
    for(var i=0; i<this.rowCount; i++) {
      if(this.__treeView__[i].mIsContainer && 
         this.__treeView__[i].mIsContainerOpen) {
        this.toggleOpenState(i);
      }
    }
    
    if(aCol.id == gXpcomTreeSort.sortedCol)
      gXpcomTreeSort.direction = (gXpcomTreeSort.direction === 0) 
          ? 1 : -gXpcomTreeSort.direction;
    
    if(aCol.id == gXpcomTreeSort.sortedCol && gXpcomTreeSort.fastIndex == this.rowCount)
    {
      this.__treeView__.reverse();
    }
    else
    {
      if(gXpcomTreeSort.sortedCol !== "") {
        var old = document.getElementById(gXpcomTreeSort.sortedCol);
        old.setAttribute("sortDirection", "");
      }
      gXpcomTreeSort.sortedCol = aCol.id;
      this.__treeView__.sort(gXpcomTreeSort.sort);
    }
    
    var sortDir = (gXpcomTreeSort.sortDirection == 1) ? "ascending" : "descending";
    aCol.element.setAttribute("sortDirection", sortDir);
    this.treebox.invalidate();
    
    if(index >= 0) {
      this.selection.select(index);
      this.treebox.ensureRowIsVisible(index);
    }
    
    gXpcomTreeSort.fastIndex = this.rowCount;
  };
  
  return treeview;
} // End of XpcomTreeViewPrototype(aTreeView)



///////////////////////////////////////////////////////////////////////////////
// Variables and objects for Components.classes tree


/**
 * Total count of Components.classes
 * @var  integer
 */
var gXpcomClassesCount = 0;


/**
 * Prototype for tree which displaying Components.classes
 * @param  array  Array of treeview items 
 */
function TreeClassesPrototype(aTreeView) 
{
  var treeview = new XpcomTreeViewPrototype(aTreeView);
  
  treeview.toggleOpenState = function (aRow) 
  {
    if (!(this.isContainer(aRow))) return;
    
    var name = this.__treeView__[aRow].mColumns["nameCol"].mCellText;
    
    if(this.isContainerOpen(aRow)) 
    {
      // we are closing opened container
      var delCount = 0;
      var level = this.getLevel (aRow);
  
      for (var i=aRow+1; i<this.rowCount; i++) {
        if (this.getLevel(i) > level) delCount++;
        else break;
      }
  
      if (delCount > 0) {
        this.__treeView__.splice(aRow + 1, delCount);
        this.treebox.rowCountChanged(aRow + 1, -delCount);
      }
  
      this.__treeView__[aRow].mIsContainerOpen = false;
    }
    else 
    {
      // we are opening selected container, so we have to know which type 
      // user select (class, interface, property or method of initialized 
      // interface).
      if (this.__treeView__[aRow].mType == "class") 
      { 
        var cl = null;
        try {
          cl = Components.classes[name].createInstance();
        } catch (e) {
          gStatusbar.update("Creating instance of '" + name + "' failed!", "error");
        }
  
        for (var ifc in Components.interfaces) 
        {
          try {
            var iface = cl.QueryInterface(Components.interfaces[ifc]);
            if (iface) 
            {
              var ifaceValue = Components.interfaces[ifc].number;
              var ifaceType = "interface";
              var ifaceItem = new XpcomTreeItemPrototype (ifc, ifaceValue, 
                  "interface", 1, false);
  
              if (ifc != "nsISupports" && ifc != "IDispatch") {
                ifaceItem.mIsContainer = true;
                ifaceItem.mIsContainerEmpty = false;
              } else {
                ifaceItem.mColumns["valueCol"].mCellText = "";
              }
  
              this.__treeView__.splice (aRow + 1, -1, ifaceItem);
              this.treebox.rowCountChanged (aRow+1, 1);
            }
          } catch (e) {}
        }
  
        this.__treeView__[aRow].mIsContainerOpen = true;
      } 
      else if (this.__treeView__[aRow].mType == "interface" &&
               name != "nsISupports") 
      {
        var clName = this.__treeView__[this.getParentIndex(aRow)].
            mColumns["nameCol"].mCellText;
        
        var inst = null;
        try {
          inst = Components.classes [clName].
              createInstance (Components.interfaces [name]);
        } catch (e) {
          try {
            inst = Components.classes [clName].
                getService (Components.interfaces [name]);
          } catch (e) { }
        }
  
        if (inst) {
          var origIndex = aRow;
  
          for (var ifaceProperty in inst) {
            var ifaceProp = inst [ifaceProperty];
            var ifacePropType = ((new String(ifaceProp)).indexOf("function")!=-1)
                ? "method" : "variable";
            var ifacePropValue = (ifacePropType == "method") ? "" : ifaceProp;
            var ifacePropTreeItem = new XpcomTreeItemPrototype (ifaceProperty, 
                ifacePropValue, ifacePropType, 2, false);
  
            this.__treeView__.splice (origIndex + 1, -1, ifacePropTreeItem);
            this.treebox.rowCountChanged (origIndex + 1, 1);
          }
  
          this.__treeView__[origIndex].mIsContainerOpen = true;
        }
      }
    }
  };
  
  return treeview;
} // End of TreeClassesPrototype(aTreeView)



///////////////////////////////////////////////////////////////////////////////
// Variables and objects for Components.interfaces tree


/**
 * Total count of Components.interfaces
 * @var  integer
 */
var gXpcomIfacesCount = 0;


/**
 * Prototype for tree which displaying Components.interfaces
 * @param  array  Array of treeview items 
 */
function TreeIfacesPrototype(aTreeView)
{
  var treeview = new XpcomTreeViewPrototype(aTreeView);
  
  treeview.toggleOpenState = function(aRow) 
  {
    if(!(this.isContainer(aRow))) return;
    
    if(this.isContainerOpen(aRow))
    {
      var delCount = 0;
      var level = this.getLevel(aRow);
  
      for (var i=aRow+1; i<this.rowCount; i++) {
        if(this.getLevel(i)>level)
          delCount++;
        else
          break;
      }
  
      if(delCount>0) {
        this.__treeView__.splice(aRow + 1, delCount);
        this.treebox.rowCountChanged(aRow + 1, -delCount);
      }
  
      this.__treeView__[aRow].mIsContainerOpen = false;
    } 
    else 
    {
      var name = this.__treeView__[aRow].mColumns["nameCol"].mCellText;
      var ifc = Components.interfaces[name];
  
      for (var ifcConst in ifc) 
      {
        var ifcConstItem = new XpcomTreeItemPrototype (ifcConst, 
            Components.interfaces [ifc][ifcConst], "constant", 1, false);
        this.__treeView__.splice(aRow+1, -1, ifcConstItem);
        this.treebox.rowCountChanged(aRow+1, 1);
      }
  
      this.__treeView__[aRow].mIsContainerOpen = true;
    }
  };
  
  return treeview;
} // End of TreeIfacesPrototype(aTreeView)



///////////////////////////////////////////////////////////////////////////////
// Variables and objects for Components.results tree


/**
 * Total count of Components.results
 * @var  integer
 */
var gXpcomResultsCount = 0;


/**
 * Prototype for tree which displaying Components.results
 * @param  array  Array of treeview items 
 */
function TreeResultsPrototype(aTreeView)
{
  var treeview = new XpcomTreeViewPrototype(aTreeView);
  
  treeview.getParentIndex = function(aRow) { return -1; };
  treeview.hasNextSibling = function(aRow, aAfterIndex) { return ((aRow+1)<this.rowCount); };
  treeview.isContainer = function(aRow) { return false; };
  treeview.isContainerOpen = function(aRow) { return false; };
  treeview.isContainerEmpty = function(aRow) { return true; };
  
  return treeview;
} // End of TreeResultsPrototype(aTreeView)



//////////////////////////////////////////////////////////////////////////
// Global variables/objects related with workspace tabbox

/**
 * Holds timer object for updateUI command.
 * @var  object
 */
var gUpdateUICmdTimer = null;

/**
 * WebProgress listener (for our lxr searchs)
 */
var gWebProgressListener =
{
  onStateChange : function(aProgress, aRequest, aFlag, aStatus) {
    if(aFlag & STATE_STOP) {
      var lxr_deck = document.getElementById("lxr_lookup-deck");
      lxr_deck.selectedIndex = 1;
      gStatusbar.update(gStrBundle.getString("pageLoadingFinished"), "warning");
      gMainContent.removeProgressListener(this);
    } 
  },
  onLocationChange : function(a,b,c){},
  onProgressChange : function(a,b,c,d,e,f){},
  onStatusChange : function(a,b,c,d){},
  onSecurityChange : function(a,b,c){},
  onLinkIconAvailable : function(a){},
  QueryInterface : function(aIID) {
    if (aIID.equals(Components.interfaces.nsIWebProgressListener) ||
        aIID.equals(Components.interfaces.nsISupportsWeakReference) ||
        aIID.equals(Components.interfaces.nsISupports))
      return this;
    
    throw Components.results.NS_NOINTERFACE;
  }
}; // End of gWebProgressListener

/**
 * Workspace statusbar's controller
 */
var gStatusbar = 
{
  mMessages : [],
  mDisplayedMessageIndex : -1,
  
  update : function(aText, aType) {
    gStatusbar.mMessages.push({ mText : aText, mType : aType});
  },
  
  nextMessage : function() {
    if((gStatusbar.mDisplayedMessageIndex + 1) >= gStatusbar.mMessages.length) return;
    
    gStatusbar.mDisplayedMessageIndex++;
    
    var msg_text = gStatusbar.mMessages[gStatusbar.mDisplayedMessageIndex].mText;
    
    if((gStatusbarLabel instanceof Components.interfaces.nsIDOMElement)) 
      gStatusbarLabel.setAttribute("label", msg_text);
    
    switch(this.mMessages[this.mDisplayedMessageIndex].mType) {
      case "debug":
        gStatusbarLabel.style.backgroundColor = "#000";
        gStatusbarLabel.style.color = "#fff";
        break;
      case "warning":
        gStatusbarLabel.style.backgroundColor = "#06a";
        gStatusbarLabel.style.color = "#fff";
        break;
      case "error":
        gStatusbarLabel.style.backgroundColor = "#f30";
        gStatusbarLabel.style.color = "#fff";
        break;
      case "success":
        gStatusbarLabel.style.backgroundColor = "#093";
        gStatusbarLabel.style.color = "#fff";
        break;
      default:
        gStatusbarLabel.style.backgroundColor = "transparent";
        gStatusbarLabel.style.color = "#000";
        break;
    }
  }
}; // End of gStatusbar


///////////////////////////////////////////////////////////////////////////////
// The base XPCOMViewer's object


/**
 * Main gXpcomViewer class.
 */
var gXpcomViewer =
{

	/**
	 * On load event handler
	 */
	onLoad : function()
	{
    window.focus();
    
    // String bundles
		gStrBundle = document.getElementById("stringBundle");
		gBrandStrBundle = document.getElementById("brandStringBundle");
    
    // UI pointers
		gXpcomTree = document.getElementById("xpcom-tree");
		gMainContent = document.getElementById("mainContent");
    gMainContentTabbox = document.getElementById("workspace-tabbox");
    gFilterToolbarTextbox = document.getElementById("main-search-textbox");
		gBookmarksTreeSplitter = document.getElementById("workspace-splitter");
    gBookmarksTree = document.getElementById("bookmarks-tree");
    gStatusbarLabel = document.getElementById("workspace-statusbar-label");
    gXpcomTreeTab = document.getElementById("xpcom-tree-tab");
    gLxrBrowserTab = document.getElementById("lxr_lookup-tab");
    gLxrBrowserCmdMenuitem = document.getElementById("showLxrSearchPage-mainmenuitem");
    gLxrBrowserCmdPopupitem = document.getElementById("showLxrSearchPage-popupmenuitem");
    gBookmarksPopupitem1 = document.getElementById("addBookmark-popupmenuitem");
    gBookmarksPopupitem2 = document.getElementById("removeBookmark-popupmenuitem");
    gBookmarksPopupSeparator = document.getElementById("bookmarksCommandsSeparator");
    gBookmarksTogglePaneMenuitem = document.getElementById("toggleBottompane-menuitem");
    gBookmarksTogglePaneMenuitemSep = document.getElementById("toggleBottompane-menuitem-sep");
    gBookmaksToolbarbutton = document.getElementById("bookmark-button");
    gBookmaksToolbarbuttonSeparator = document.getElementById("toolbarAddBookmarkSeparator");
    gBookmarksFilterToolbarbutton = document.getElementById("showWithBookmark-button");
    gBookmarksFilterToolbarbuttonSep = document.getElementById("filterBookmarksSeparator");
    gBookmarksMainTreeCol = document.getElementById("bookmarkCol");
    gBookmarksMainTreeColSep = document.getElementById("bookmarkCol-sep");
    
    gStartupTreeViewName = ((kPrefs.getBoolPref("useLastView")) &&
                            (kPrefs.prefHasUserValue("useLastViewName")))
        ? kPrefs.getCharPref("useLastViewName")
        : "classes";
    
    // Get "human" names for single tree views
    kTreeViewsItemNames.classes = gStrBundle.getString("classesTreeViewName");
    kTreeViewsItemNames.ifaces = gStrBundle.getString("ifacesTreeViewName");
    kTreeViewsItemNames.results = gStrBundle.getString("resultsTreeViewName");
    
    // Initialize bookmarks service
    try {
      gBookmarksSrv = new BookmarksServicePrototype("bookmarks.xml");
      gAllowBookmarks = true
    } catch(e) {
      gAllowBookmarks = false;
      gStatusbar.update(gStrBundle.getString("storageServiceInitError"), "error");
    }
    
    // Register XPCOMViewer commands
    gSupportedCommands = new Object();
    gSupportedCommands.cmd_quit               = gCmdQuit;
    gSupportedCommands.cmd_updateui           = gCmdUpdateUI;
    gSupportedCommands.cmd_fullscreen         = gCmdFullScreen;
    gSupportedCommands.cmd_showAbout          = gCmdShowAbout;
    gSupportedCommands.cmd_showHomePage       = gCmdShowHomePage;
    gSupportedCommands.cmd_showHelp           = gCmdShowHelp;
    gSupportedCommands.cmd_showJSConsole      = gCmdShowJsConsole;
    gSupportedCommands.cmd_showPreferences    = gCmdShowPreferences;
    gSupportedCommands.cmd_saveAsXML          = gCmdSaveReportAs;
    gSupportedCommands.cmd_saveAsHTML         = gCmdSaveReportAs;
    gSupportedCommands.cmd_saveAsTXT          = gCmdSaveReportAs;
    gSupportedCommands.cmd_showClasses        = gCmdShowView;
    gSupportedCommands.cmd_showIfaces         = gCmdShowView;
    gSupportedCommands.cmd_showResults        = gCmdShowView;
    gSupportedCommands.cmd_showFiltered       = gCmdShowFiltered;
    gSupportedCommands.cmd_showAll            = gCmdShowAll;
    gSupportedCommands.cmd_ifaceLXRLookup     = gCmdLXRLookup;
    gSupportedCommands.cmd_showLXRSearch      = gCmdShowLXRSearchPage;
    gSupportedCommands.cmd_closeLxrTab        = gCmdCloseLXRTab;
    gSupportedCommands.cmd_copyNameOfSel      = gCmdCopyNameOfSel;
    gSupportedCommands.cmd_generateCppCode    = gCmdGenerateCode;
    gSupportedCommands.cmd_generateJsCode     = gCmdGenerateCode;
    gSupportedCommands.cmd_generatePythonCode = gCmdGenerateCode;
    
    if(gAllowBookmarks) {
      // These commands are related bookmarks only
      gSupportedCommands.cmd_showWithBookmark = gCmdShowWithBookmark;
      gSupportedCommands.cmd_addBookmark      = gCmdAddBookmark;
      gSupportedCommands.cmd_editBookmark     = gCmdEditBookmark;
      gSupportedCommands.cmd_removeBookmark   = gCmdRemoveBookmark;
      gSupportedCommands.cmd_toggleBottompane = gCmdToggleBottompane;
    }
    
    window.controllers.appendController(gXpcomViewerController);
    
    this.loadAllViews();
    this.refreshTree(gStartupTreeViewName);
    
    // XXX Add observer for preferences (if bookmarks are allowed,
    //     if additional tree styles have be used etc.)
    // XXX kObserverSrv.addObserver(this, "fullScreenModeChanged", true);
    
    // Periodically start the update UI command (But run it first, because
    // that delay is unpleasant for users).
    this.doCommand("cmd_updateui");
    if(gUpdateUICmdTimer == null) {
      gUpdateUICmdTimer = new TimerPrototype();
      gUpdateUICmdTimer.init(gXpcomViewer.updateUI, 500,
                             gUpdateUICmdTimer.TYPE_REPEATING_PRECISE);
    }
    
    window.focus();
	}, // end onLoad()


	/**
	 * On unload event handler
	 */
	onUnLoad : function()
	{
    gUpdateUICmdTimer.mTimer.cancel();
    gUpdateUICmdTimer = null;
    
    return true;
	}, // end onUnLoad()


	/**
	 * On close event handler
	 */
	onClose : function()
	{
	  if((kPrefs.getBoolPref("useLastView"))) {
      kPrefs.setCharPref("useLastViewName", gCurrentTreeView);
    }
    
    if(gAllowBookmarks) {
      gBookmarksSrv.close();
      gBookmarksSrv = null;
    }
    
	  return true;
	}, // end onClose()
  
  
  /**
   * Fired when main tree context menu is showing
   * @param  nsIDOMElement
   */
  onXpcomTreePopupShowing : function(aPopupElm)
  {
    if(gAllowBookmarks) {
      if(!(aPopupElm instanceof Components.interfaces.nsIDOMElement)) return;
      if(gXpcomTree.currentIndex == -1) return;
      
      var sel_item = this.getSelectedTreeItem();
      var menuitem1 = document.getElementById("addBookmark-popupmenuitem");
      var menuitem2 = document.getElementById("removeBookmark-popupmenuitem");
      
      if(gBookmarksSrv.hasItemBookmark(sel_item.mName)) {
        menuitem1.disabled = true;
        menuitem2.disabled = false;
      } else {
        menuitem1.disabled = false;
        menuitem2.disabled = true;
      }
    }
  }, // end onXpcomTreePopupShowing(aPopupElm)
  
  
  /**
   * Runs UI update command (is a callback for gUpdateUICmdTimer)
   */
  updateUI : function()
  {
    gXpcomViewer.doCommand("cmd_updateui");
  }, // end updateUI()
  
  
  /**
   * Execute XPCOMViewer command
   * @param string
   */
  doCommand : function(aCmd)
  {
    var disp = document.commandDispatcher;
    var cont = disp.getControllerForCommand(aCmd);
    
    if(!cont) {
      gStatusbar.update(gStrBundle.getFormattedString("doCommandContError", [aCmd]), "warning");
      return;
    }
    
    cont.doCommand(aCmd);
  }, // end doCommand(aCmd)
  
  
  /**
   * Returns name of selected item (e.g. "@mozilla.org/..." etc.).
   * @return object
   */
  getSelectedTreeItem : function()
  {
    if(gXpcomTree.currentIndex == -1) {
      return new SelectedTreeItemPrototype(-1, "", "");
    }
    
    var itemIdx = gXpcomTree.currentIndex;
    var itemName = gXpcomTree.view.getCellText(itemIdx, gXpcomTree.columns.getPrimaryColumn());
    var itemType = "";
    
    switch(gCurrentTreeView)
    {
      case "classes":
        if(gXpcomTree.view.getLevel(itemIdx) === 0) itemType = "class";
        else if(gXpcomTree.view.getLevel(itemIdx) == 1) itemType = "interface";
        else if(gXpcomTree.view.getLevel(itemIdx) == 2) itemType = "property";
        break;
      
      case "ifaces":
        itemType = (gXpcomTree.view.getLevel(itemIdx) === 0)
            ? "interface"
            : "ifaceConstant";
        break;
      
      case "results":
        itemType = "result";
        break;
    }
    
    return new SelectedTreeItemPrototype(itemIdx, itemName, itemType);
  }, // end getSelectedTreeItem()
	
	
	/**
	 * Load all views and store them into gXpcomTreeViewBackup
	 * XXX Any UI messages about loading progress
	 */
	loadAllViews : function()
	{
    // Create gXpcomTreeViewBackup.classes
  	gXpcomClassesCount = 0;
		var newTreeView = [];
	  for (var cl in Components.classes) 
	  {
	    var cls = Components.classes[cl];
      
	    newTreeView.push (new XpcomTreeItemPrototype(cls.name, cls.number, 
	    		"class", 0, true));
	    newTreeView[gXpcomClassesCount].mIsContainerEmpty = false;
	    gXpcomClassesCount++;
	  }
	  gXpcomTreeViewBackup.classes = newTreeView;
    
    // Create gXpcomTreeViewBackup.ifaces
		gXpcomIfacesCount = 0;
		newTreeView = [];
		for (var ifc in Components.interfaces) 
		{
		  var iface = Components.interfaces[ifc];
		
		  newTreeView.push (new XpcomTreeItemPrototype(iface.name, iface.number,
		      "interface", 0, false));
		
		      var addIndex = 0;
		  for (var ifaceItem in iface) addIndex++;
		
		  if (addIndex > 0) {
		    newTreeView[gXpcomIfacesCount].mIsContainer = true;
		    newTreeView[gXpcomIfacesCount].mIsContainerEmpty = false;
		  }
		
		  gXpcomIfacesCount++;
		}
		gXpcomTreeViewBackup.ifaces = newTreeView;
    
    // Create gXpcomTreeViewBackup.results
		gXpcomResultsCount = 0;
		newTreeView = [];
    for (var res in Components.results) 
    {
      newTreeView.push (new XpcomTreeItemPrototype (res, 
      		Components.results[res], "result", 0, false));

      gXpcomResultsCount++;
    }
 		gXpcomTreeViewBackup.results = newTreeView;
	}, // end loadAllViews()


	/**
	 * Refresh tree with given reference uri.
	 *
	 * @param  string
	 */
	refreshTree : function(aTreeType)
	{
    gCurrentTreeView = aTreeType;

    // Remove filter if any applied and restore treeview from backup
    if (gXpcomTreeFilter != false)
      goDoCommand("cmd_showAll");
    
    var restoredTreeView = gXpcomTreeViewBackup[aTreeType];
    
    switch(aTreeType) {
      case "classes":
        gXpcomTree.view = new TreeClassesPrototype(restoredTreeView);
        break;
      
      case "ifaces":
        gXpcomTree.view = new TreeIfacesPrototype(restoredTreeView);
        break;
      
      case "results":
        gXpcomTree.view = new TreeResultsPrototype(restoredTreeView);
        break;
    }
    gXpcomTree.treeBoxObject.view.selection.clearSelection();
    
    // Update user interface
    gStatusbar.update(gStrBundle.getFormattedString("loadingTreeFinished", 
        [kTreeViewsNames[gCurrentTreeView], gXpcomTree.view.rowCount]), "success");
	} // end refreshTree(aTreeType)

}; // end of gXpcomViewer

